//   Copyright Naoki Shibata and contributors 2010 - 2025.
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#include <stdint.h>
#include <assert.h>
#include <signal.h>
#include <setjmp.h>

#include "misc.h"

#define SLEEF_IMPORT_IS_EXPORT
#include "sleefquad.h"

#include "qdispatcher.h"

NOEXPORT Sleef_quadx2 sleef_cpuid_EXT;

#ifdef ENABLE_AVX2
static void tryEXT1() { sleef_cpuid_EXT = Sleef_sinq2_u10avx2128(sleef_cpuid_EXT); }
#define SUBST_IF_EXT1(funcExt1) if (cpuSupportsExt(tryEXT1)) p = funcExt1;
#endif

#ifdef ENABLE_VSX3
static void tryEXT1() { sleef_cpuid_EXT = Sleef_sinq2_u10vsx3(sleef_cpuid_EXT); }
#define SUBST_IF_EXT1(funcExt1) if (cpuSupportsExt(tryEXT1)) p = funcExt1;
#endif

#ifdef ENABLE_VXE2
static void tryEXT1() { sleef_cpuid_EXT = Sleef_sinq2_u10vxe2(sleef_cpuid_EXT); }
#define SUBST_IF_EXT1(funcExt1) if (cpuSupportsExt(tryEXT1)) p = funcExt1;
#endif

#ifndef SUBST_IF_EXT1
#define SUBST_IF_EXT1(funcExt1)
#endif

//

DISPATCH_vq_vq_vq(Sleef_quadx2, Sleef_addq2_u05, pnt_addq2_u05, disp_addq2_u05, Sleef_addq2_u05advsimd, Sleef_addq2_u05x)
DISPATCH_vq_vq_vq(Sleef_quadx2, Sleef_subq2_u05, pnt_subq2_u05, disp_subq2_u05, Sleef_subq2_u05advsimd, Sleef_subq2_u05x)
DISPATCH_vq_vq_vq(Sleef_quadx2, Sleef_mulq2_u05, pnt_mulq2_u05, disp_mulq2_u05, Sleef_mulq2_u05advsimd, Sleef_mulq2_u05x)
DISPATCH_vq_vq_vq(Sleef_quadx2, Sleef_divq2_u05, pnt_divq2_u05, disp_divq2_u05, Sleef_divq2_u05advsimd, Sleef_divq2_u05x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_negq2, pnt_negq2, disp_negq2, Sleef_negq2_advsimd, Sleef_negq2_x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_sqrtq2_u05, pnt_sqrtq2_u05, disp_sqrtq2_u05, Sleef_sqrtq2_u05advsimd, Sleef_sqrtq2_u05x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_cbrtq2_u10, pnt_cbrtq2_u10, disp_cbrtq2_u10, Sleef_cbrtq2_u10advsimd, Sleef_cbrtq2_u10x)
DISPATCH_vx_vq_vq(Sleef_quadx2, int32x2_t, Sleef_icmpltq2, pnt_icmpltq2, disp_icmpltq2, Sleef_icmpltq2_advsimd, Sleef_icmpltq2_x)
DISPATCH_vx_vq_vq(Sleef_quadx2, int32x2_t, Sleef_icmpleq2, pnt_icmpleq2, disp_icmpleq2, Sleef_icmpleq2_advsimd, Sleef_icmpleq2_x)
DISPATCH_vx_vq_vq(Sleef_quadx2, int32x2_t, Sleef_icmpgtq2, pnt_icmpgtq2, disp_icmpgtq2, Sleef_icmpgtq2_advsimd, Sleef_icmpgtq2_x)
DISPATCH_vx_vq_vq(Sleef_quadx2, int32x2_t, Sleef_icmpgeq2, pnt_icmpgeq2, disp_icmpgeq2, Sleef_icmpgeq2_advsimd, Sleef_icmpgeq2_x)
DISPATCH_vx_vq_vq(Sleef_quadx2, int32x2_t, Sleef_icmpeqq2, pnt_icmpeqq2, disp_icmpeqq2, Sleef_icmpeqq2_advsimd, Sleef_icmpeqq2_x)
DISPATCH_vx_vq_vq(Sleef_quadx2, int32x2_t, Sleef_icmpneq2, pnt_icmpneq2, disp_icmpneq2, Sleef_icmpneq2_advsimd, Sleef_icmpneq2_x)
DISPATCH_vx_vq_vq(Sleef_quadx2, int32x2_t, Sleef_icmpq2, pnt_icmpq2, disp_icmpq2, Sleef_icmpq2_advsimd, Sleef_icmpq2_x)
DISPATCH_vx_vq_vq(Sleef_quadx2, int32x2_t, Sleef_iunordq2, pnt_iunordq2, disp_iunordq2, Sleef_iunordq2_advsimd, Sleef_iunordq2_x)
DISPATCH_vx_vq(Sleef_quadx2, float64x2_t, Sleef_cast_to_doubleq2, pnt_cast_to_doubleq2, disp_cast_to_doubleq2, Sleef_cast_to_doubleq2_advsimd, Sleef_cast_to_doubleq2_x)
DISPATCH_vq_vx(Sleef_quadx2, float64x2_t, Sleef_cast_from_doubleq2, pnt_cast_from_doubleq2, disp_cast_from_doubleq2, Sleef_cast_from_doubleq2_advsimd, Sleef_cast_from_doubleq2_x)
DISPATCH_vx_vq(Sleef_quadx2, int64x2_t, Sleef_cast_to_int64q2, pnt_cast_to_int64q2, disp_cast_to_int64q2, Sleef_cast_to_int64q2_advsimd, Sleef_cast_to_int64q2_x)
DISPATCH_vq_vx(Sleef_quadx2, int64x2_t, Sleef_cast_from_int64q2, pnt_cast_from_int64q2, disp_cast_from_int64q2, Sleef_cast_from_int64q2_advsimd, Sleef_cast_from_int64q2_x)
DISPATCH_vx_vq(Sleef_quadx2, uint64x2_t, Sleef_cast_to_uint64q2, pnt_cast_to_uint64q2, disp_cast_to_uint64q2, Sleef_cast_to_uint64q2_advsimd, Sleef_cast_to_uint64q2_x)
DISPATCH_vq_vx(Sleef_quadx2, uint64x2_t, Sleef_cast_from_uint64q2, pnt_cast_from_uint64q2, disp_cast_from_uint64q2, Sleef_cast_from_uint64q2_advsimd, Sleef_cast_from_uint64q2_x)
DISPATCH_q_vq_vx(Sleef_quadx2, int, Sleef_getq2, pnt_getq2, disp_getq2, Sleef_getq2_advsimd, Sleef_getq2_x)
DISPATCH_vq_vq_vi_q(Sleef_quadx2, int, Sleef_setq2, pnt_setq2, disp_setq2, Sleef_setq2_advsimd, Sleef_setq2_x)
DISPATCH_vq_vx(Sleef_quadx2, Sleef_quad, Sleef_splatq2, pnt_splatq2, disp_splatq2, Sleef_splatq2_advsimd, Sleef_splatq2_x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_sinq2_u10, pnt_sinq2_u10, disp_sinq2_u10, Sleef_sinq2_u10advsimd, Sleef_sinq2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_cosq2_u10, pnt_cosq2_u10, disp_cosq2_u10, Sleef_cosq2_u10advsimd, Sleef_cosq2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_tanq2_u10, pnt_tanq2_u10, disp_tanq2_u10, Sleef_tanq2_u10advsimd, Sleef_tanq2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_asinq2_u10, pnt_asinq2_u10, disp_asinq2_u10, Sleef_asinq2_u10advsimd, Sleef_asinq2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_acosq2_u10, pnt_acosq2_u10, disp_acosq2_u10, Sleef_acosq2_u10advsimd, Sleef_acosq2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_atanq2_u10, pnt_atanq2_u10, disp_atanq2_u10, Sleef_atanq2_u10advsimd, Sleef_atanq2_u10x)
DISPATCH_vq_vq_vq(Sleef_quadx2, Sleef_atan2q2_u10, pnt_atan2q2_u10, disp_atan2q2_u10, Sleef_atan2q2_u10advsimd, Sleef_atan2q2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_expq2_u10, pnt_expq2_u10, disp_expq2_u10, Sleef_expq2_u10advsimd, Sleef_expq2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_exp2q2_u10, pnt_exp2q2_u10, disp_exp2q2_u10, Sleef_exp2q2_u10advsimd, Sleef_exp2q2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_exp10q2_u10, pnt_exp10q2_u10, disp_exp10q2_u10, Sleef_exp10q2_u10advsimd, Sleef_exp10q2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_expm1q2_u10, pnt_expm1q2_u10, disp_expm1q2_u10, Sleef_expm1q2_u10advsimd, Sleef_expm1q2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_logq2_u10, pnt_logq2_u10, disp_logq2_u10, Sleef_logq2_u10advsimd, Sleef_logq2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_log2q2_u10, pnt_log2q2_u10, disp_log2q2_u10, Sleef_log2q2_u10advsimd, Sleef_log2q2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_log10q2_u10, pnt_log10q2_u10, disp_log10q2_u10, Sleef_log10q2_u10advsimd, Sleef_log10q2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_log1pq2_u10, pnt_log1pq2_u10, disp_log1pq2_u10, Sleef_log1pq2_u10advsimd, Sleef_log1pq2_u10x)
DISPATCH_vq_vq_vq(Sleef_quadx2, Sleef_powq2_u10, pnt_powq2_u10, disp_powq2_u10, Sleef_powq2_u10advsimd, Sleef_powq2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_sinhq2_u10, pnt_sinhq2_u10, disp_sinhq2_u10, Sleef_sinhq2_u10advsimd, Sleef_sinhq2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_coshq2_u10, pnt_coshq2_u10, disp_coshq2_u10, Sleef_coshq2_u10advsimd, Sleef_coshq2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_tanhq2_u10, pnt_tanhq2_u10, disp_tanhq2_u10, Sleef_tanhq2_u10advsimd, Sleef_tanhq2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_asinhq2_u10, pnt_asinhq2_u10, disp_asinhq2_u10, Sleef_asinhq2_u10advsimd, Sleef_asinhq2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_acoshq2_u10, pnt_acoshq2_u10, disp_acoshq2_u10, Sleef_acoshq2_u10advsimd, Sleef_acoshq2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_atanhq2_u10, pnt_atanhq2_u10, disp_atanhq2_u10, Sleef_atanhq2_u10advsimd, Sleef_atanhq2_u10x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_truncq2, pnt_truncq2, disp_truncq2, Sleef_truncq2_advsimd, Sleef_truncq2_x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_floorq2, pnt_floorq2, disp_floorq2, Sleef_floorq2_advsimd, Sleef_floorq2_x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_ceilq2, pnt_ceilq2, disp_ceilq2, Sleef_ceilq2_advsimd, Sleef_ceilq2_x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_roundq2, pnt_roundq2, disp_roundq2, Sleef_roundq2_advsimd, Sleef_roundq2_x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_rintq2, pnt_rintq2, disp_rintq2, Sleef_rintq2_advsimd, Sleef_rintq2_x)
DISPATCH_vq_vq(Sleef_quadx2, Sleef_fabsq2, pnt_fabsq2, disp_fabsq2, Sleef_fabsq2_advsimd, Sleef_fabsq2_x)
DISPATCH_vq_vq_vq(Sleef_quadx2, Sleef_copysignq2, pnt_copysignq2, disp_copysignq2, Sleef_copysignq2_advsimd, Sleef_copysignq2_x)
DISPATCH_vq_vq_vq(Sleef_quadx2, Sleef_fmaxq2, pnt_fmaxq2, disp_fmaxq2, Sleef_fmaxq2_advsimd, Sleef_fmaxq2_x)
DISPATCH_vq_vq_vq(Sleef_quadx2, Sleef_fminq2, pnt_fminq2, disp_fminq2, Sleef_fminq2_advsimd, Sleef_fminq2_x)
DISPATCH_vq_vq_vq(Sleef_quadx2, Sleef_fdimq2_u05, pnt_fdimq2_u05, disp_fdimq2_u05, Sleef_fdimq2_u05advsimd, Sleef_fdimq2_u05x)
DISPATCH_vq_vq_vq(Sleef_quadx2, Sleef_fmodq2, pnt_fmodq2, disp_fmodq2, Sleef_fmodq2_advsimd, Sleef_fmodq2_x)
DISPATCH_vq_vq_vq(Sleef_quadx2, Sleef_remainderq2, pnt_remainderq2, disp_remainderq2, Sleef_remainderq2_advsimd, Sleef_remainderq2_x)
DISPATCH_vq_vq_pvx(Sleef_quadx2, int32x2_t, Sleef_frexpq2, pnt_frexpq2, disp_frexpq2, Sleef_frexpq2_advsimd, Sleef_frexpq2_x)
DISPATCH_vq_vq_pvx(Sleef_quadx2, Sleef_quadx2, Sleef_modfq2, pnt_modfq2, disp_modfq2, Sleef_modfq2_advsimd, Sleef_modfq2_x)
DISPATCH_vq_vq_vq(Sleef_quadx2, Sleef_hypotq2_u05, pnt_hypotq2_u05, disp_hypotq2_u05, Sleef_hypotq2_u05advsimd, Sleef_hypotq2_u05x)
DISPATCH_vq_vq_vx(Sleef_quadx2, int32x2_t, Sleef_ldexpq2, pnt_ldexpq2, disp_ldexpq2, Sleef_ldexpq2_advsimd, Sleef_ldexpq2_x)
DISPATCH_vx_vq(Sleef_quadx2, int32x2_t, Sleef_ilogbq2, pnt_ilogbq2, disp_ilogbq2, Sleef_ilogbq2_advsimd, Sleef_ilogbq2_x)
DISPATCH_vq_vq_vq_vq(Sleef_quadx2, Sleef_fmaq2_u05, pnt_fmaq2_u05, disp_fmaq2_u05, Sleef_fmaq2_u05advsimd, Sleef_fmaq2_u05x)
